/*
 * aconversion.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

#include "ConversionHandler.h"

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Converts between regular expressions, regular grammars and finite automaton.", ' ', "0.01" );

		std::vector < std::string > formalisms {
			"fa", "re", "rg", "pda"
		};
		TCLAP::ValuesConstraint < std::string > allowedFormalisms ( formalisms );
		TCLAP::ValueArg < std::string > target ( "t", "target", "Specifies target formalism", true, "", & allowedFormalisms );
		cmd.add ( target );

		std::vector < std::string > algorithms {
			"algebraic", "elimination", "brzozowski", "glushkov", "thompson", "incoming", "outgoing", "bottomup", "topdown", "default", "glushkovrtenaive", "glushkovrte"
		};
		TCLAP::ValuesConstraint < std::string > allowedAlgorithms ( algorithms );
		TCLAP::ValueArg < std::string > algorithm ( "a", "algorithm", "Specifies algorithm to use", false, "default", & allowedAlgorithms );
		cmd.add ( algorithm );

		TCLAP::ValueArg < std::string > input ( "i", "input", "input to convert", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		cli::Parser parser ( cli::Lexer ( "execute <#stdin > $input" ) );
		parser.parse ( )->run ( environment );
		std::string inputType = environment.getVariable ( "input" )->getReturnType ( );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		ConversionHandler conv( inputType, target.getValue(), algorithm.getValue() );
		std::string convertString = conv.convert( );
		parser = cli::Parser ( cli::Lexer ( convertString ) );
		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		parser = cli::Parser ( cli::Lexer ( "execute $output >#stdout" ) );
		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::end();

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch(const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		common::Streams::err << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		common::Streams::err << "Unknown exception caught." << std::endl;
		return 127;
	}
}
