/*
 * AllEpsilonClosure.h
 *
 *  Created on: 12. 4. 2015
 *	  Author: Jan Travnicek
 */

#ifndef ALL_EPSILON_CLOSURE_H_
#define ALL_EPSILON_CLOSURE_H_

#include <set>
#include <map>
#include <deque>
#include <iostream>

#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <regexp/properties/RegExpEpsilon.h>

namespace automaton {

namespace properties {

class AllEpsilonClosure {
public:
	/**
	 * Computes allEpsilon closure of a state in allEpsilon nonfree automaton
	 */
	template < class SymbolType, class EpsilonType, class StateType >
	static ext::map<StateType, ext::set<StateType>> allEpsilonClosure( const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm);
	template < class SymbolType, class StateType >
	static ext::map<StateType, ext::set<StateType>> allEpsilonClosure( const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm);
	template < class SymbolType, class StateType >
	static ext::map<StateType, ext::set<StateType>> allEpsilonClosure( const automaton::NFA < SymbolType, StateType > & fsm);
	template < class SymbolType, class StateType >
	static ext::map<StateType, ext::set<StateType>> allEpsilonClosure( const automaton::DFA < SymbolType, StateType > & fsm);
	template < class SymbolType, class StateType >
	static ext::map<StateType, ext::set<StateType>> allEpsilonClosure( const automaton::ExtendedNFA < SymbolType, StateType > & fsm);
	template < class SymbolType, class StateType >
	static ext::map<StateType, ext::set<StateType>> allEpsilonClosure( const automaton::CompactNFA < SymbolType, StateType > & fsm);
};

template < class SymbolType, class EpsilonType, class StateType >
ext::map<StateType, ext::set<StateType>> AllEpsilonClosure::allEpsilonClosure( const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm) {
	ext::deque<ext::map<StateType, ext::set<StateType>>> Qi;

	Qi.push_back({});
	for(const StateType& state : fsm.getStates())
		Qi.at(0)[state].insert(state);

	int i = 1;
	while(true) {
		Qi.push_back( Qi.at( i - 1 ) );

		for( const std::pair<const StateType, ext::set<StateType>> & p : Qi.at( i - 1 ) )
			for(const StateType& pclosure : p.second )
				for(const auto& transition : fsm.getEpsilonTransitionsFromState(pclosure)) {
					Qi.at( i )[p.first].insert(transition.second.begin(), transition.second.end());
				}

		if( Qi.at( i ) == Qi.at( i - 1 ) )
			break;

		i = i + 1;
	};

	return Qi.at(i);
}

template < class SymbolType, class StateType >
ext::map<StateType, ext::set<StateType>> AllEpsilonClosure::allEpsilonClosure( const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm) {
	ext::map<StateType, ext::set<StateType>> closure;
	for(const StateType& state : fsm.getStates())
		closure[state].insert(state);
	return closure;
}

template < class SymbolType, class StateType >
ext::map<StateType, ext::set<StateType>> AllEpsilonClosure::allEpsilonClosure( const automaton::NFA < SymbolType, StateType > & fsm) {
	ext::map<StateType, ext::set<StateType>> closure;
	for(const StateType& state : fsm.getStates())
		closure[state].insert(state);
	return closure;
}

template < class SymbolType, class StateType >
ext::map<StateType, ext::set<StateType>> AllEpsilonClosure::allEpsilonClosure( const automaton::DFA < SymbolType, StateType > & fsm) {
	ext::map<StateType, ext::set<StateType>> closure;
	for(const StateType& state : fsm.getStates())
		closure[state].insert(state);
	return closure;
}

template < class SymbolType, class StateType >
ext::map<StateType, ext::set<StateType>> AllEpsilonClosure::allEpsilonClosure( const automaton::ExtendedNFA < SymbolType, StateType > & fsm) {
	ext::map<StateType, ext::set<StateType>> res;
	ext::map<StateType, ext::set<StateType>> step;

	for(const std::pair<const std::pair<StateType, regexp::UnboundedRegExpStructure < SymbolType > >, ext::set<StateType> >& transition : fsm.getTransitions() )
		if( regexp::properties::RegExpEpsilon::languageContainsEpsilon( transition.first.second ) )
			step[transition.first.first].insert(transition.second.begin(), transition.second.end());

	for(const StateType& state : fsm.getStates())
		step[state].insert(state);

	do {
		res = step;

		for(const std::pair<const StateType, ext::set<StateType>>& item : res)
			for(const StateType& to : item.second)
				step[item.first].insert(res[to].begin(), res[to].end());
	} while(res != step);

	return res;
}

template < class SymbolType, class StateType >
ext::map<StateType, ext::set<StateType>> AllEpsilonClosure::allEpsilonClosure( const automaton::CompactNFA < SymbolType, StateType > & fsm) {
	ext::map<StateType, ext::set<StateType>> res;
	ext::map<StateType, ext::set<StateType>> step;

	for(const std::pair<const std::pair<StateType, ext::vector < SymbolType > >, ext::set<StateType> >& transition : fsm.getTransitions() )
		if( transition.first.second.size() == 0 )
			step[transition.first.first].insert(transition.second.begin(), transition.second.end());

	for(const StateType& state : fsm.getStates())
		step[state].insert(state);

	do {
		res = step;

		for(const std::pair<const StateType, ext::set<StateType>>& item : res)
			for(const StateType& to : item.second)
				step[item.first].insert(res[to].begin(), res[to].end());
	} while(res != step);

	return res;
}

} /* namespace properties */

} /* namespace automaton */

#endif /* ALL_EPSILON_CLOSURE_H_ */
