/*
 * Rename.cpp
 *
 *  Created on: Dec 9, 2013
 *	Author: Jan Travnicek
 */

#include "Rename.h"

#include <map>
#include <vector>
#include <algorithm>
#include <sstream>
#include <iostream>
#include <registration/AlgoRegistration.hpp>

namespace automaton {

namespace simplify {

automaton::DFA<> Rename::rename(const automaton::DFA<>& fsm) {
	int counter = 0;
	ext::map<DefaultStateType, int > renamingData;

	for(auto iter = fsm.getStates().begin(); iter != fsm.getStates().end(); iter++) {
		renamingData.insert(std::make_pair(*iter, counter++));
	}

	automaton::DFA<> result(DefaultStateType(renamingData.find(fsm.getInitialState())->second));

	result.setInputAlphabet(fsm.getInputAlphabet());

	for(auto iter = fsm.getStates().begin(); iter != fsm.getStates().end(); iter++) {
		result.addState(DefaultStateType(renamingData.find(*iter)->second));
	}

	for(auto iter = fsm.getFinalStates().begin(); iter != fsm.getFinalStates().end(); iter++) {
		result.addFinalState(DefaultStateType(renamingData.find(*iter)->second));
	}

	for(auto iter = fsm.getTransitions().begin(); iter != fsm.getTransitions().end(); iter++) {
		result.addTransition(DefaultStateType(renamingData.find(iter->first.first)->second), iter->first.second, DefaultStateType(renamingData.find(iter->second)->second));
	}

	return result;
}

auto RenameDFA = registration::AbstractRegister < Rename, automaton::DFA < >, const automaton::DFA < > & > ( Rename::rename );

automaton::DPDA < > Rename::rename(const automaton::DPDA < > & pda) {
	int counterState = 0;
	ext::map<DefaultStateType, int > renamingDataState;
	int counterSymbol = 0;
	ext::map<DefaultSymbolType, int > renamingDataSymbol;

	for(const DefaultStateType & state : pda.getStates())
		renamingDataState.insert(std::make_pair(state, counterState++));

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		renamingDataSymbol.insert(std::make_pair(symbol, counterSymbol++));

	automaton::DPDA < > result(DefaultStateType(renamingDataState.find(pda.getInitialState())->second), DefaultSymbolType(renamingDataSymbol.find(pda.getInitialSymbol())->second));

	result.setInputAlphabet(pda.getInputAlphabet());

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		result.addPushdownStoreSymbol(DefaultSymbolType(renamingDataSymbol.find(symbol)->second));

	for(const DefaultStateType & state : pda.getStates())
		result.addState(DefaultStateType(renamingDataState.find(state)->second));

	for(const DefaultStateType & state : pda.getFinalStates())
		result.addFinalState(DefaultStateType(renamingDataState.find(state)->second));

	for(const auto & transition : pda.getTransitions()) {
		ext::vector<DefaultSymbolType> pop;
		for(const auto& elem : std::get<2>(transition.first)) {
			pop.push_back(DefaultSymbolType(renamingDataSymbol.find(elem)->second));
		}
		ext::vector<DefaultSymbolType> push;
		for(const auto& elem : transition.second.second) {
			push.push_back(DefaultSymbolType(renamingDataSymbol.find(elem)->second));
		}
		result.addTransition(DefaultStateType(renamingDataState.find(std::get<0>(transition.first))->second), std::get<1>(transition.first), pop, DefaultStateType(renamingDataState.find(transition.second.first)->second), push);
	}

	return result;
}

auto RenameDPDA = registration::AbstractRegister < Rename, automaton::DPDA < >, const automaton::DPDA < > & > ( Rename::rename );

automaton::SinglePopDPDA < > Rename::rename(const automaton::SinglePopDPDA < > & pda) {
	int counterState = 0;
	ext::map<DefaultStateType, int > renamingDataState;
	int counterSymbol = 0;
	ext::map<DefaultSymbolType, int > renamingDataSymbol;

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		renamingDataState.insert(std::make_pair(*iter, counterState++));
	}

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		renamingDataSymbol.insert(std::make_pair(symbol, counterSymbol++));

	automaton::SinglePopDPDA < > result(DefaultStateType(renamingDataState.find(pda.getInitialState())->second), DefaultSymbolType(renamingDataSymbol.find(pda.getInitialSymbol())->second));

	result.setInputAlphabet(pda.getInputAlphabet());

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		result.addPushdownStoreSymbol(DefaultSymbolType(renamingDataSymbol.find(symbol)->second));

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		result.addState(DefaultStateType(renamingDataState.find(*iter)->second));
	}

	for(auto iter = pda.getFinalStates().begin(); iter != pda.getFinalStates().end(); iter++) {
		result.addFinalState(DefaultStateType(renamingDataState.find(*iter)->second));
	}

	for(auto iter = pda.getTransitions().begin(); iter != pda.getTransitions().end(); iter++) {
		ext::vector<DefaultSymbolType> push;
		for(const auto& elem : iter->second.second) {
			push.push_back(DefaultSymbolType(renamingDataSymbol.find(elem)->second));
		}
		result.addTransition(DefaultStateType(renamingDataState.find(std::get<0>(iter->first))->second), std::get<1>(iter->first), DefaultSymbolType(renamingDataSymbol.find(std::get<2>(iter->first))->second), DefaultStateType(renamingDataState.find(iter->second.first)->second), push);
	}

	return result;
}

auto RenameSinglePopDPDA = registration::AbstractRegister < Rename, automaton::SinglePopDPDA < >, const automaton::SinglePopDPDA < > & > ( Rename::rename );

automaton::InputDrivenDPDA < > Rename::rename(const automaton::InputDrivenDPDA < > & pda) {
	int counter = 0;
	ext::map<DefaultStateType, int > renamingData;
	int counterSymbol = 0;
	ext::map<DefaultSymbolType, int > renamingDataSymbol;

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		renamingData.insert(std::make_pair(*iter, counter++));
	}

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		renamingDataSymbol.insert(std::make_pair(symbol, counterSymbol++));

	automaton::InputDrivenDPDA < > result(DefaultStateType(renamingData.find(pda.getInitialState())->second), DefaultSymbolType(renamingDataSymbol.find(pda.getInitialSymbol())->second));

	result.setInputAlphabet(pda.getInputAlphabet());

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		result.addPushdownStoreSymbol(DefaultSymbolType(renamingDataSymbol.find(symbol)->second));

	for(const std::pair<const DefaultSymbolType, std::pair<ext::vector<DefaultSymbolType>, ext::vector<DefaultSymbolType>>> operation : pda.getPushdownStoreOperations()) {
		ext::vector<DefaultSymbolType> pop;
		for(const auto& elem : operation.second.first) {
			pop.push_back(DefaultSymbolType(renamingDataSymbol.find(elem)->second));
		}
		ext::vector<DefaultSymbolType> push;
		for(const auto& elem : operation.second.second) {
			push.push_back(DefaultSymbolType(renamingDataSymbol.find(elem)->second));
		}
		result.setPushdownStoreOperation(operation.first, pop, push);
	}

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		result.addState(DefaultStateType(renamingData.find(*iter)->second));
	}

	for(auto iter = pda.getFinalStates().begin(); iter != pda.getFinalStates().end(); iter++) {
		result.addFinalState(DefaultStateType(renamingData.find(*iter)->second));
	}

	for(auto iter = pda.getTransitions().begin(); iter != pda.getTransitions().end(); iter++) {
		result.addTransition(DefaultStateType(renamingData.find(iter->first.first)->second), iter->first.second, DefaultStateType(renamingData.find(iter->second)->second));
	}

	return result;
}

auto RenameInputDrivenDPDA = registration::AbstractRegister < Rename, automaton::InputDrivenDPDA < >, const automaton::InputDrivenDPDA < > & > ( Rename::rename );

automaton::VisiblyPushdownDPDA < > Rename::rename(const automaton::VisiblyPushdownDPDA < > & pda) {
	int counterState = 0;
	ext::map<DefaultStateType, int > renamingDataState;
	int counterSymbol = 0;
	ext::map<DefaultSymbolType, int > renamingDataSymbol;

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		renamingDataState.insert(std::make_pair(*iter, counterState++));
	}

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		renamingDataSymbol.insert(std::make_pair(symbol, counterSymbol++));

	automaton::VisiblyPushdownDPDA < > result(DefaultStateType(renamingDataState.find(pda.getInitialState())->second), DefaultSymbolType(renamingDataSymbol.find(pda.getBottomOfTheStackSymbol())->second));

	result.setCallInputAlphabet(pda.getCallInputAlphabet());
	result.setLocalInputAlphabet(pda.getLocalInputAlphabet());
	result.setReturnInputAlphabet(pda.getReturnInputAlphabet());

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		result.addPushdownStoreSymbol(DefaultSymbolType(renamingDataSymbol.find(symbol)->second));

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		result.addState(DefaultStateType(renamingDataState.find(*iter)->second));
	}

	for(auto iter = pda.getFinalStates().begin(); iter != pda.getFinalStates().end(); iter++) {
		result.addFinalState(DefaultStateType(renamingDataState.find(*iter)->second));
	}

	for(auto iter = pda.getCallTransitions().begin(); iter != pda.getCallTransitions().end(); iter++) {
		result.addCallTransition(DefaultStateType(renamingDataState.find(iter->first.first)->second), iter->first.second, DefaultStateType(renamingDataState.find(iter->second.first)->second), DefaultSymbolType(renamingDataSymbol.find(iter->second.second)->second));
	}

	for(auto iter = pda.getLocalTransitions().begin(); iter != pda.getLocalTransitions().end(); iter++) {
		result.addLocalTransition(DefaultStateType(renamingDataState.find(iter->first.first)->second), iter->first.second, DefaultStateType(renamingDataState.find(iter->second)->second));
	}

	for(auto iter = pda.getReturnTransitions().begin(); iter != pda.getReturnTransitions().end(); iter++) {
		result.addReturnTransition(DefaultStateType(renamingDataState.find(std::get<0>(iter->first))->second), std::get<1>(iter->first), DefaultSymbolType(renamingDataSymbol.find(std::get<2>(iter->first))->second), DefaultStateType(renamingDataState.find(iter->second)->second));
	}

	return result;
}

auto RenameVisiblyPushdownDPDA = registration::AbstractRegister < Rename, automaton::VisiblyPushdownDPDA < >, const automaton::VisiblyPushdownDPDA < > & > ( Rename::rename );

automaton::RealTimeHeightDeterministicDPDA < > Rename::rename(const automaton::RealTimeHeightDeterministicDPDA < > & pda) {
	int counterState = 0;
	ext::map<DefaultStateType, int > renamingDataState;
	int counterSymbol = 0;
	ext::map<DefaultSymbolType, int > renamingDataSymbol;

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		renamingDataState.insert(std::make_pair(*iter, counterState++));
	}

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		renamingDataSymbol.insert(std::make_pair(symbol, counterSymbol++));

	automaton::RealTimeHeightDeterministicDPDA < > result(DefaultStateType(renamingDataState.find(pda.getInitialState())->second), DefaultSymbolType(renamingDataSymbol.find(pda.getBottomOfTheStackSymbol())->second));

	result.setInputAlphabet(pda.getInputAlphabet());

	for(const DefaultSymbolType & symbol : pda.getPushdownStoreAlphabet())
		result.addPushdownStoreSymbol(DefaultSymbolType(renamingDataSymbol.find(symbol)->second));

	for(auto iter = pda.getStates().begin(); iter != pda.getStates().end(); iter++) {
		result.addState(DefaultStateType(renamingDataState.find(*iter)->second));
	}

	for(auto iter = pda.getFinalStates().begin(); iter != pda.getFinalStates().end(); iter++) {
		result.addFinalState(DefaultStateType(renamingDataState.find(*iter)->second));
	}

	for(auto iter = pda.getCallTransitions().begin(); iter != pda.getCallTransitions().end(); iter++) {
		result.addCallTransition(DefaultStateType(renamingDataState.find(iter->first.first)->second), iter->first.second, DefaultStateType(renamingDataState.find(iter->second.first)->second), DefaultSymbolType(renamingDataSymbol.find(iter->second.second)->second));
	}

	for(auto iter = pda.getLocalTransitions().begin(); iter != pda.getLocalTransitions().end(); iter++) {
		result.addLocalTransition(DefaultStateType(renamingDataState.find(iter->first.first)->second), iter->first.second, DefaultStateType(renamingDataState.find(iter->second)->second));
	}

	for(auto iter = pda.getReturnTransitions().begin(); iter != pda.getReturnTransitions().end(); iter++) {
		result.addReturnTransition(DefaultStateType(renamingDataState.find(std::get<0>(iter->first))->second), std::get<1>(iter->first), DefaultSymbolType(renamingDataSymbol.find(std::get<2>(iter->first))->second), DefaultStateType(renamingDataState.find(iter->second)->second));
	}

	return result;
}

auto RenameRealTimeHeightDeterministicDPDA = registration::AbstractRegister < Rename, automaton::RealTimeHeightDeterministicDPDA < >, const automaton::RealTimeHeightDeterministicDPDA < > & > ( Rename::rename );

} /* namespace simplify */

} /* namespace automaton */
