/*
 * ToCNF.cpp
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#include "ToCNF.h"

#include "EpsilonRemover.h"
#include "SimpleRulesRemover.h"
#include <common/DefaultSymbolsPairType.h>
#include <exception/CommonException.h>
#include <common/createUnique.hpp>
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace simplify {

template<class T>
ext::pair<DefaultSymbolType, DefaultSymbolType> splitToPairs(T& grammar, const ext::vector<DefaultSymbolType>& rhs, unsigned from, unsigned size, ext::map<DefaultSymbolType, DefaultSymbolType>& createdSymbols) {
	if(size == 2) {
		return ext::make_pair(rhs[from], rhs[from + 1]);
	} else if(size == 3) {
		DefaultSymbolType firstLhs {rhs[from]};

		auto second = splitToPairs(grammar, rhs, from + 1, 2, createdSymbols);
		DefaultSymbolType secondProposal{DefaultSymbolsPairType(second)};
		if(!createdSymbols.count(secondProposal)) {
			createdSymbols.insert(std::make_pair(secondProposal, common::createUnique(secondProposal, grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet())));
		}
		grammar.addNonterminalSymbol(createdSymbols.find(secondProposal)->second);
		grammar.addRawRule(createdSymbols.find(secondProposal)->second, {std::move(second.first), std::move(second.second)});

		return ext::make_pair(std::move(firstLhs), createdSymbols.find(secondProposal)->second);
	} else {
		auto first = splitToPairs(grammar, rhs, from, size / 2, createdSymbols);
		DefaultSymbolType firstProposal{DefaultSymbolsPairType(first)};
		if(!createdSymbols.count(firstProposal)) {
			createdSymbols.insert(std::make_pair(firstProposal, common::createUnique(firstProposal, grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet())));
		}
		grammar.addNonterminalSymbol(createdSymbols.find(firstProposal)->second);
		grammar.addRawRule(createdSymbols.find(firstProposal)->second, {std::move(first.first), std::move(first.second)});

		auto second = splitToPairs(grammar, rhs, from + size / 2, size - size / 2, createdSymbols);
		DefaultSymbolType secondProposal{DefaultSymbolsPairType(second)};
		if(!createdSymbols.count(secondProposal)) {
			createdSymbols.insert(std::make_pair(secondProposal, common::createUnique(secondProposal, grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet())));
		}
		grammar.addNonterminalSymbol(createdSymbols.find(secondProposal)->second);
		grammar.addRawRule(createdSymbols.find(secondProposal)->second, {std::move(second.first), std::move(second.second)});

		return ext::make_pair(createdSymbols.find(firstProposal)->second, createdSymbols.find(secondProposal)->second);
	}
}

template<class T>
grammar::CNF < > convertInternal( const T & origGrammar ) {
	T grammarTmp(origGrammar.getInitialSymbol());

	grammarTmp.setNonterminalAlphabet(origGrammar.getNonterminalAlphabet() );
	grammarTmp.setTerminalAlphabet( origGrammar.getTerminalAlphabet() );

	ext::map<DefaultSymbolType, DefaultSymbolType> createdSymbols;
	auto origRules = origGrammar.getRawRules();
	for( const auto & origRule : origRules ) {
		for( const auto& origRhs : origRule.second ) {
			if(origRhs.size() == 1 || origRhs.size() == 2)
				grammarTmp.addRawRule(origRule.first, origRhs);
			else if(origRhs.size() > 2) {
				auto second = splitToPairs(grammarTmp, origRhs, 0, origRhs.size(), createdSymbols);
				grammarTmp.addRawRule(origRule.first, {std::move(second.first), std::move(second.second)});
			} else
				throw exception::CommonException("Invalid rule in grammar");
		}
	}

	grammarTmp.setGeneratesEpsilon(origGrammar.getGeneratesEpsilon());

	grammar::CNF < > grammar(grammarTmp.getInitialSymbol());

	grammar.setNonterminalAlphabet( grammarTmp.getNonterminalAlphabet() );
	grammar.setTerminalAlphabet( grammarTmp.getTerminalAlphabet() );

	ext::map<DefaultSymbolType, DefaultSymbolType> terminalToShadowNonterminal;
	for( const auto & symbol : grammarTmp.getTerminalAlphabet() ) {
		DefaultSymbolType shadowSymbol = common::createUnique(symbol, grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		terminalToShadowNonterminal.insert( std::make_pair( symbol, shadowSymbol ));
		grammar.addNonterminalSymbol( shadowSymbol );
		grammar.addRule(std::move(shadowSymbol), symbol);
	}

	auto tmpRules = grammarTmp.getRawRules();
	for( const auto & tmpRule : tmpRules ) {
		for( const auto& tmpRhs : tmpRule.second ) {
			if(tmpRhs.size() == 2) {
				if(grammarTmp.getNonterminalAlphabet().count(tmpRhs[0])) {
					if(grammarTmp.getNonterminalAlphabet().count(tmpRhs[1])) {
						grammar.addRawRule(tmpRule.first, tmpRhs);
					} else {
						grammar.addRawRule(tmpRule.first, {tmpRhs[0], terminalToShadowNonterminal.find(tmpRhs[1])->second});
					}
				} else {
					if(grammarTmp.getNonterminalAlphabet().count(tmpRhs[1])) {
						grammar.addRawRule(tmpRule.first, {terminalToShadowNonterminal.find(tmpRhs[0])->second, tmpRhs[1]});
					} else {
						grammar.addRawRule(tmpRule.first, {terminalToShadowNonterminal.find(tmpRhs[0])->second, terminalToShadowNonterminal.find(tmpRhs[1])->second });
					}
				}
			} else // tmpRhs.size() == 1
				grammar.addRawRule(tmpRule.first, tmpRhs);
		}
	}

	grammar.setGeneratesEpsilon(grammarTmp.getGeneratesEpsilon());

	return grammar;
}

grammar::CNF < > ToCNF::convert(const grammar::CFG < > & origGrammar) {
	return convertInternal(grammar::simplify::SimpleRulesRemover::remove(grammar::simplify::EpsilonRemover::remove(origGrammar)));
}

grammar::CNF < > ToCNF::convert(const grammar::EpsilonFreeCFG < > & origGrammar) {
	return convertInternal(grammar::simplify::SimpleRulesRemover::remove(origGrammar));
}

grammar::CNF < > ToCNF::convert(const grammar::CNF < > & origGrammar) {
	return origGrammar;
}

grammar::CNF < > ToCNF::convert(const grammar::GNF < > & origGrammar) {
	return convertInternal(origGrammar);
}

grammar::CNF < > ToCNF::convert(const grammar::LG < > & origGrammar) {
	return convertInternal(grammar::simplify::SimpleRulesRemover::remove(grammar::simplify::EpsilonRemover::remove(origGrammar)));
}

grammar::CNF < > ToCNF::convert(const grammar::LeftLG< > & origGrammar) {
	return convertInternal(grammar::simplify::SimpleRulesRemover::remove(grammar::simplify::EpsilonRemover::remove(origGrammar)));
}

grammar::CNF < > ToCNF::convert(const grammar::LeftRG < > & origGrammar) {
	return convertInternal(origGrammar);
}

grammar::CNF < > ToCNF::convert(const grammar::RightLG < > & origGrammar) {
	return convertInternal(grammar::simplify::SimpleRulesRemover::remove(grammar::simplify::EpsilonRemover::remove(origGrammar)));
}

grammar::CNF < > ToCNF::convert(const grammar::RightRG < > & origGrammar) {
	return convertInternal(origGrammar);
}

auto ToCNFCFG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::CFG < > & > ( ToCNF::convert );
auto ToCNFEpsilonFreeCFG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::EpsilonFreeCFG < > & > ( ToCNF::convert );
auto ToCNFCNF = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::CNF < > & > ( ToCNF::convert );
auto ToCNFGNF = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::GNF < > & > ( ToCNF::convert );
auto ToCNFLG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::LG < > & > ( ToCNF::convert );
auto ToCNFLeftLG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::LeftLG < > & > ( ToCNF::convert );
auto ToCNFLeftRG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::LeftRG < > & > ( ToCNF::convert );
auto ToCNFRightLG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::RightLG < > & > ( ToCNF::convert );
auto ToCNFRightRG = registration::AbstractRegister < ToCNF, grammar::CNF < >, const grammar::RightRG < > & > ( ToCNF::convert );

} /* namespace simplify */

} /* namespace grammar */
