/*
 * Author: Radovan Cerveny
 */

#ifndef STRINGOLOGY_BACKWARD_DAWG_MATCHING_H__
#define STRINGOLOGY_BACKWARD_DAWG_MATCHING_H__

#include <set>

#include <automaton/FSM/DFA.h>
#include <string/LinearString.h>

#include <stringology/properties/BackboneLength.h>

namespace stringology {

namespace query {

/**
 * Implementation of Backward DAWG Matching.
 */
class BackwardDAWGMatching {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	template < class SymbolType, class StateType >
	static ext::set < unsigned > match ( const string::LinearString < SymbolType > & subject, const automaton::DFA < SymbolType, StateType > & suffixAutomaton );

};

template < class SymbolType, class StateType >
ext::set < unsigned > BackwardDAWGMatching::match ( const string::LinearString < SymbolType > & subject, const automaton::DFA < SymbolType, StateType > & suffixAutomaton ) {
	ext::set < unsigned > occ;

	size_t patternSize = stringology::properties::BackboneLength::length ( suffixAutomaton );
	size_t subjectSize = subject.getContent ( ).size ( );

	bool fail;
	size_t posInSubject = 0;

	while ( posInSubject <= subjectSize - patternSize ) {
		StateType currentState = suffixAutomaton.getInitialState ( );

		size_t posInPattern = patternSize;
		size_t lastPrefixPos = posInPattern;

		fail = false;
		while ( posInPattern > 0 && ! fail ) {
			auto transition = suffixAutomaton.getTransitions ( ).find ( { currentState, subject.getContent ( ).at ( posInSubject + posInPattern - 1 ) } );

			if ( transition == suffixAutomaton.getTransitions ( ).end ( ) )
				fail = true;
			else
				currentState = transition->second;

			posInPattern--;

			 // found a prefix of nonreversed pattern that does not correspond to the entire pattern
			if ( ( posInPattern != 0 ) && ( suffixAutomaton.getFinalStates ( ).find ( currentState ) != suffixAutomaton.getFinalStates ( ).end ( ) ) )
				lastPrefixPos = posInPattern;
		}

		if ( ! fail )
			 // Yay, there is match!!!
			occ.insert ( posInSubject );

		posInSubject += lastPrefixPos;
	}

	return occ;
}

} /* namespace query */

} /* namespace stringology */

#endif /* STRINGOLOGY_BACKWARD_DAWG_MATCHING_H__ */
