/*
 * UnboundedRegExpConcatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_CONCATENATION_H_
#define UNBOUNDED_REG_EXP_CONCATENATION_H_

#include <vector>
#include <sstream>

#include <exception/CommonException.h>

#include "UnboundedRegExpElement.h"

namespace regexp {

/**
 * Represents concatenation operator in the regular expression. Contains list of UnboundedRegExpElement
 * as operands of the operator.
 */
template < class SymbolType >
class UnboundedRegExpConcatenation : public UnboundedRegExpElement < SymbolType >, public ext::VararyNode < ext::smart_ptr < UnboundedRegExpElement < SymbolType > >, ext::smart_ptr < const UnboundedRegExpElement < SymbolType > >, UnboundedRegExpConcatenation < SymbolType > > {
public:
	void accept ( typename UnboundedRegExpElement < SymbolType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit UnboundedRegExpConcatenation ( );

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	UnboundedRegExpConcatenation < SymbolType > * clone ( ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	UnboundedRegExpConcatenation < SymbolType > * plunder ( ) && override;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	ext::smart_ptr < FormalRegExpElement < SymbolType > > asFormal ( ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	bool testSymbol ( const SymbolType & symbol ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( ext::set < SymbolType > & alphabet ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::checkAlphabet()
	 */
	bool checkAlphabet ( const ext::set < SymbolType > & alphabet ) const override;

	/**
	 * @return elements
	 */
	const ext::vector < ext::smart_ptr < const UnboundedRegExpElement < SymbolType > > > & getElements ( ) const;

	/**
	 * @return elements
	 */
	const ext::vector < ext::smart_ptr < UnboundedRegExpElement < SymbolType > > > & getElements ( );

	/**
	 * @param element to append
	 */
	void appendElement ( UnboundedRegExpElement < SymbolType > && element );
	void appendElement ( const UnboundedRegExpElement < SymbolType > & element );

	int compare ( const UnboundedRegExpElement < SymbolType > & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	int compare ( const UnboundedRegExpConcatenation & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	void operator >>( std::ostream & out ) const override;

	explicit operator std::string ( ) const override;

	virtual ext::smart_ptr < UnboundedRegExpElement < DefaultSymbolType > > normalize ( ) && override {
		UnboundedRegExpConcatenation < DefaultSymbolType > * res = new UnboundedRegExpConcatenation < DefaultSymbolType > ( );

		for ( ext::smart_ptr < UnboundedRegExpElement < SymbolType > > & element : this->getChildren ( ) )
			res->appendElement ( std::move ( * std::move ( * element ).normalize ( ) ) );

		return ext::smart_ptr < UnboundedRegExpElement < DefaultSymbolType > > ( res );
	}
};

} /* namespace regexp */

#include "../formal/FormalRegExpConcatenation.h"
#include "../formal/FormalRegExpEpsilon.h"

namespace regexp {

template < class SymbolType >
UnboundedRegExpConcatenation < SymbolType >::UnboundedRegExpConcatenation ( ) {
}

template < class SymbolType >
const ext::vector < ext::smart_ptr < const UnboundedRegExpElement < SymbolType > > > & UnboundedRegExpConcatenation < SymbolType >::getElements ( ) const {
	return this->getChildren();
}

template < class SymbolType >
const ext::vector < ext::smart_ptr < UnboundedRegExpElement < SymbolType > > > & UnboundedRegExpConcatenation < SymbolType >::getElements ( ) {
	return this->getChildren();
}

template < class SymbolType >
void UnboundedRegExpConcatenation < SymbolType >::appendElement ( UnboundedRegExpElement < SymbolType > && element ) {
	this->pushBackChild ( ext::smart_ptr < UnboundedRegExpElement < SymbolType > > ( std::move ( element ).plunder ( ) ) );
}

template < class SymbolType >
void UnboundedRegExpConcatenation < SymbolType >::appendElement ( const UnboundedRegExpElement < SymbolType > & element ) {
	this->appendElement ( ext::move_copy ( element ) );
}

template < class SymbolType >
UnboundedRegExpConcatenation < SymbolType > * UnboundedRegExpConcatenation < SymbolType >::clone ( ) const {
	return new UnboundedRegExpConcatenation ( * this );
}

template < class SymbolType >
UnboundedRegExpConcatenation < SymbolType > * UnboundedRegExpConcatenation < SymbolType >::plunder ( ) && {
	return new UnboundedRegExpConcatenation ( std::move ( * this ) );
}

template < class SymbolType >
ext::smart_ptr < FormalRegExpElement < SymbolType > > UnboundedRegExpConcatenation < SymbolType >::asFormal ( ) const {
	if ( getElements ( ).size ( ) == 0 ) return ext::smart_ptr < FormalRegExpElement < SymbolType > > ( new FormalRegExpEpsilon < SymbolType > ( ) );

	ext::smart_ptr < FormalRegExpElement < SymbolType > > res = getElements ( )[getElements ( ).size ( ) - 1]->asFormal ( );

	for ( unsigned i = getElements ( ).size ( ) - 1; i >= 1; i-- )
		res = ext::smart_ptr < FormalRegExpElement < SymbolType > > ( new FormalRegExpConcatenation < SymbolType > ( std::move ( * getElements ( )[i - 1]->asFormal ( ) ), std::move ( * res ) ) );

	return res;
}

template < class SymbolType >
int UnboundedRegExpConcatenation < SymbolType >::compare ( const UnboundedRegExpConcatenation < SymbolType > & other ) const {
	static ext::compare < typename std::decay < decltype ( getElements ( ) ) >::type > comp;

	return comp ( getElements ( ), other.getElements ( ) );
}

template < class SymbolType >
void UnboundedRegExpConcatenation < SymbolType >::operator >>( std::ostream & out ) const {
	out << "(UnboundedRegExpConcatenation";

	for ( const auto & child : getElements() )
		out << " " << * child;

	out << ")";
}

template < class SymbolType >
bool UnboundedRegExpConcatenation < SymbolType >::testSymbol ( const SymbolType & symbol ) const {
	for ( const auto & child : getElements() )
		if ( child->testSymbol ( symbol ) ) return true;

	return false;
}

template < class SymbolType >
bool UnboundedRegExpConcatenation < SymbolType >::checkAlphabet ( const ext::set < SymbolType > & alphabet ) const {
	for ( const auto & child : getElements() )
		if ( !child->checkAlphabet ( alphabet ) ) return false;

	return true;
}

template < class SymbolType >
void UnboundedRegExpConcatenation < SymbolType >::computeMinimalAlphabet ( ext::set < SymbolType > & alphabet ) const {
	for ( const auto & child : getElements() )
		child->computeMinimalAlphabet ( alphabet );
}

template < class SymbolType >
UnboundedRegExpConcatenation < SymbolType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_CONCATENATION_H_ */
